const express = require("express");
const bodyParser = require("body-parser");
const readline = require("readline");
const { db } = require("./firebaseAdmin");
const admin = require('firebase-admin');
const app = express();
const port = 3000; // You can choose any port

app.use(bodyParser.json());
const dgram = require('dgram');

// Function to send UDP messages
function sendUdpMessage(message, port) {
  const client = dgram.createSocket('udp4');
  const messageBuffer = Buffer.from(message);

  client.send(messageBuffer, 0, messageBuffer.length, port, 'localhost', (err) => {
    client.close();
    if (err) {
      console.error('UDP message send error:', err);
    } else {
      console.log(`UDP message sent to port ${port}:`, message);
    }
  });
}
// Create interface for readline
const rl = readline.createInterface({
  input: process.stdin,
  output: process.stdout
});

let userId = "";
let plantName = "";

// Function to deactivate a plant
const deactivatePlant = async (uid, pname) => {
  try {
    const plantsQuerySnapshot = await db.collection('plants')
      .where('userId', '==', uid.trim())
      .where('Name', '==', pname.trim())
      .get();

    if (!plantsQuerySnapshot.empty) {
      const batch = db.batch();

      plantsQuerySnapshot.forEach((doc) => {
        batch.update(doc.ref, { isActive: false });
      });

      await batch.commit();
      console.log('Successfully updated isActive to false for all matching plants.');
    } else {
      console.log('No matching plant document found to deactivate.');
    }
  } catch (error) {
    console.error('Error updating Firestore', error);
  }
};

// Endpoint to handle updates
app.post("/updatePlant", async (req, res) => {
  const { data } = req.body;
  // Assuming that data contains the userId and plantName to identify the plant document
  try {
    const plantDoc = db.collection("plants").doc(`${data.userId}_${data.plantName}`);
    const doc = await plantDoc.get();

    if (!doc.exists) {
      throw new Error("No matching document found!");
    }

    // Perform the update
    await plantDoc.update(data);
    res.send({
      status: "success",
      message: "Plant data updated successfully"
    });
  } catch (error) {
    console.error("Error updating Firestore", error);
    res.status(500).send({ status: "error", message: error.message });
  }
});

app.post('/deactivatePlant', async (req, res) => {
  // Assuming that the request body will contain the userId and plantName
  const { userId, plantName } = req.body;
  try {
    await deactivatePlant(userId, plantName);
    res.json({ message: 'Deactivation attempted.' });
  } catch (error) {
    res.status(500).json({ message: 'Error deactivating plant.', error: error.message });
  }
});

// Prompt user for initial data
rl.question("Enter user ID: ", function (uid) {
  userId = uid;
  rl.question("Enter plant name: ", function (pname) {
    plantName = pname;
    rl.close();

    // Start the server after getting the data
    app.listen(port, () => {
      console.log(`Server running on http://localhost:${port}`);
      console.log(`Monitoring updates for user ID: ${userId} and plant name: ${plantName}`);

      // Periodically check the plant's active status
      setInterval(() => checkAndDeactivatePlant(userId, plantName), 6000); // 6000 milliseconds = 1 minute
    });
  });
});

async function checkAndDeactivatePlant(uid, pname) {
  const plantsQuerySnapshot = await db.collection("plants")
    .where("userId", "==", uid.trim())
    .where("Name", "==", pname.trim())
    .get();

  if (!plantsQuerySnapshot.empty) {
    const plantDoc = plantsQuerySnapshot.docs[0];
    const plantData = plantDoc.data();

    if (plantData.isActive) {
      console.log("Plant is active:", plantData.portion);

      if (plantData.portion == 'small') {
        // Action for small portion
        console.log('Taking action for small portion');
        sendUdpMessage("smallWatering", 12345);
      } else if (plantData.portion == 'medium') {
        console.log('Taking action for medium portion');
        sendUdpMessage("mediumWatering", 12345);
      } else {

        console.log('Taking action for large portion sizes');

        sendUdpMessage("largeWatering", 12345);
      }
      await deactivatePlant(uid, pname);
    } else {
      console.log("Plant watering is not active.");

    }
  } else {
    console.log("No matching plant document found./n shutting down ");
    sendUdpMessage("shutDown", 12345);
  }
}
// Function to add a new value to the dataPoints array for a plant
const addNewValueToDataPoints = async (uid, pname, newValue) => {
  try {
    const plantsQuerySnapshot = await db.collection('plants')
      .where('userId', '==', uid.trim())
      .where('Name', '==', pname.trim())
      .get();

    if (!plantsQuerySnapshot.empty) {
      const plantDocRef = plantsQuerySnapshot.docs[0].ref;

      // Create a new object to add to the dataPoints array
      // Generate a timestamp using the JavaScript Date object
      const newPoint = {
        value: newValue,
        timestamp: new Date() // This will use your server's local time
      };

      await plantDocRef.update({
        dataPoints: admin.firestore.FieldValue.arrayUnion(newPoint)
      });

      console.log(`Successfully added new data point with value ${newValue} to plant ${pname}.`);
    } else {
      console.log('No matching plant document found to add a new data point.');
    }
  } catch (error) {
    console.error('Error adding new data point to Firestore', error);
  }
};


// Endpoint to add a new value to a plant's dataPoints
app.post('/addValueToDataPoints', async (req, res) => {
  const { userId, plantName, value } = req.body; // Assume these values are provided in the request body

  try {
    await addNewValueToDataPoints(userId, plantName, value);
    res.json({ message: 'New data point added to plant.' });
  } catch (error) {
    res.status(500).json({ message: 'Error adding new data point to plant.', error: error.message });
  }
});

/*
Using Postman
Open Postman.
Create a new request.
Set the request type to POST.
Enter the URL http://localhost:3000/addValueToDataPoints .
Go to the "Body" tab.
Select the "raw" option and choose JSON from the dropdown.
Enter the JSON data:
{
  "userId": "kH36yNARnoUDLHIiasr7c9bErE03",
  "plantName": "Samfisher",
  "value": 600
}


*/